
# q-expansion of newform 144.12.a.p, downloaded from the LMFDB on 24 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 144
weight = 12
poly_data = [-27, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-192, 384], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [-3934, -4], [-45528, 14], [79540, 231], [525238, -316], [-715442, -3608], [10933300, -1529], [17903368, 8246], [114413850, 19852], [-32361056, -9544], [37779390, 27188], [-600607098, 215696], [22759916, 108017], [-614539632, 272140], [1904274962, 1171660], [-3006463292, -1435821], [4894896454, -1529564], [-7351547612, -7105861], [2159995544, 6699714], [5527819738, 1086456], [-25978811632, 13791148], [-54113987956, 4517417], [-35594145930, 30252936], [-849903838, 21399800], [106164817794, -30878100], [-26674886392, 16740790], [13407152340, -62843585], [-63793647914, -91489580], [-166050153426, -69403712], [411600944768, 123704224], [-331390871140, -38442347], [-362047564698, 344544592], [-82553600052, -491329319], [-837755895566, 122285884], [135059652472, 516220586], [-118438604186, 243039108], [662406719748, 625793091], [-170625773640, -56792934], [-2220931930518, 29120284], [-3266916205908, -673220863], [-1833626633874, 887637796], [-1468064412096, -429449104], [3568634818370, -535726408], [-637883345310, -2743409956], [-6823043163160, -149435298], [2876070121460, 2378769911], [-4448445493408, 941595400], [8245639158204, 1641417357], [12149846719486, -1685513180], [1119442305350, 4427989832], [-2849517808432, -3510437252], [4871181329490, -1701329512], [-3171536779516, -1266997629], [-7301187963650, 5141964800], [3809887845592, 12783005298], [-14820455967926, -12853710580], [10283934962896, -16721536580], [-28082353396082, 5109005748], [19317940963126, 240298856], [15376232371612, 5875369621], [24415488581442, 2703615004], [36620100537300, -7055040961], [13811172043368, -17549987394], [22315945231146, 1010827248], [5387545186746, 31636913420], [501431605132, -9468319735], [-5674330347150, 53039713888], [-112073612870364, 15885889243], [49547356978854, -33856208556], [24408773417182, 10605524768], [-131372216198664, 42540029322], [-1863162160720, -67240795100], [71214375912878, 10710453012], [-111364335131780, -23547240675], [-24871259952000, -89088856992], [-58034803698142, 124757052812], [-68300987233738, -141455547564], [30114777459918, -6398767704], [-111138769262262, 33425513648], [-222907668646404, 56718146717], [148372558333502, 42242883396], [118462870934288, -10171842772], [-81412767904622, 112664605272], [60863222277144, -88241604318], [-210064390799548, -70731520685], [-432994655406658, 142956504264], [-475612698720934, 59959120816], [-298927683145462, -273117604020], [365689206288016, -94881082548], [-12116520483060, -31540719191], [135108912776096, -271633219528], [-274331498697592, -311758556522], [-355579776154284, 102285995455], [481126890995604, 110253529967], [1296515599776936, -23619513298], [-30475621542534, 148527729468], [294894463251430, 333689279760], [-261339746219188, -91562668967], [-130230290273562, 523240449060], [-1253091551415164, -246505866173], [858937068443114, -64861002948], [-1738106274128084, -16658739967], [-1632238377492010, -551423309808], [-178330487438532, -430919406323], [1574346087244994, 159704134880], [1827102379576180, -369495818889], [1782340294025102, 365244822336], [222332987606472, 52873874438], [-1538127868557686, -304515682696], [888874797214432, -24168880152], [-411713349059458, -253952283452], [1791520248001734, -455712298328], [-1626979808573140, -550841544831], [1848967722373592, 131878507698], [-2599973563685506, 536899458312], [-1515392436397980, -56226426069], [1613899345060184, 423289093330], [1784800855693002, -638197998900], [-1140460273591476, 2172590964537], [-210155119980274, 2102214794788], [-2712319338156062, 505954474424], [-3006533258809150, 396357916684], [895743236649364, -1582200793873], [2341028117860180, -1398391245441], [-2698470051066054, -2775793821668], [441443294775710, -2544607087372], [1353773194884208, -1687583606924], [1316093286508728, -4420746754854], [-183591045890266, -2733021612236], [-4563583123492796, -2509743117], [-4101891520457096, -2541012912214], [-3753544345283280, 740703275396], [-8601707497754578, -642493834108], [4694017254829462, -1556180419824], [-2061411823085566, -2123654473032], [-9871110970777182, 1695789052492], [-1361409547835596, 20381045735], [-3026035083440614, 35035611580], [6917972741862790, -4668648808432], [-13621048640000788, 942646935249], [-2887475255289582, -4219446881300], [-2266681823023208, 891938483394], [-4739349230424124, -2843011871245], [4311014318317126, 7402144964164], [11446785854446488, 2640818617538], [2613072328130894, -7554340580940], [-9407331656491146, 6620958455952], [4851243687566556, 1594854943525], [6284728207597600, 4288540913944], [4584076520666070, -4663879220812], [-11464368132761810, -3702925301600], [-7380329105986796, -9983845210129], [-17158078497588952, 1242599580878], [-122732928868660, 814827515481], [19218209289087664, -1918330537116], [-4731820415400328, -1890917047446], [-1887807701781218, 7915918502856], [-7084048080498438, -9026886549192], [21791006540912490, -1646142685204], [22582783405203372, 1419917452513], [2301285475012950, 7550645537232], [11383027040469736, -17923291279586], [-6075750963818508, -16453113348553], [17011948645798926, 2519487828200], [-35252018983785912, 7017007682790], [-7269901828026784, 18465938686216], [36607141063696126, 7756915239540], [-20408368004999854, -22342658185960], [-21323249927784238, -343516224164], [2248232485059588, -2355980453533], [33868005641443974, 11202738574356], [-46675629171878440, -9966327981134], [5872299230213914, 15528349160856], [-2594151559943216, -25050482093700], [-7231049609730506, 10734852610280], [-38924275280973668, 2073775382581], [-22051099817957310, 23143787911468], [-64053499312269880, -356239369722], [-10805380540040170, -5276408303324], [-45220682591271104, -3693867544080], [-46774472118909220, -18169813357467], [21131765083312158, 24754460211908], [5865761451911718, 27814830447464], [-72582121604646672, -1828577911020], [-16149299912259150, -19917392211140], [-5794953789174362, -20613619451516], [2926826403624900, -9580039504173], [39054419388349498, -2640035090896], [-43603900592052864, 18321790725664], [-11616993349165694, 34358918689120], [-2024294428908492, 34306091923495], [24514165359594164, 22385280730407], [-25803301273965670, 35311749619916], [-84578450868530436, 4632318253885], [-95809440061263866, -3739088375128], [-30348175295980910, 48829629929920], [92319391340030662, 4922045329796], [13144442032810942, 77361140093056], [-19147278469564392, 29685158371042], [-42603670645240310, 13171113295228], [-18397087207701104, 63072260562188], [4337905075761614, -26553911314860], [16457635316994466, -6873664766464], [27991331384959572, -95382361485601], [-21998524091501958, -57899037852564], [18253385822699776, 62089830272192], [37182789017874204, 7929848763221], [41747563278930406, -44600107863416], [116014656578552396, -17802586479399], [-67961851536453198, 56309171868824], [-31329790002673806, 1860302916124], [35736483508312568, -50558241515446], [65090326880476772, 66437006568107], [51931332039191864, -24272122591174], [196905571707531386, 33143860509552], [90287845629651568, 43713709294836], [282763101787630478, -26171010474328], [7821566146375368, -17313350767930], [-106981345742018214, -43229531254196], [240261773264312702, 22581630580964], [254423760416004312, -52433978939022], [-56236508605860738, 121900101125768], [-92600821314594992, -27561112866980], [206620441787967564, -18366242939303], [-114517257833748466, 122916035786516], [185862823531168182, -53656430895864], [123404718795021024, 161564595534440], [-9443184048559800, 83076191006822], [-39922899693393260, -28529777061905], [453549802769638038, 16095135912884], [-157763865118752172, 23926635661407], [-65773443783558464, -184675134689136], [-21226382554140250, -24189952556952], [-67220143398004212, -102085608226231], [86789239877729648, 66327566857396], [-199811440597669390, -46681338777000], [263437456381890866, 45697409267804], [-90927122545165660, 197036274316315], [66954632509239160, -111395326236438], [225918051789364332, -120559680786671], [80508132907767292, 178992504682589], [110489449590891560, 127613251377102], [-453293475493344842, -44621442189212], [-165458992522937778, 57565410050592], [153531607393089800, 41444219743606], [-260676473152887904, -125965430094280], [-215731047154815108, 140178429828221], [362091510646128620, 125208710077745], [-61299452213426032, 110862684581964], [269994606561753414, -145892509601580], [433681834910842686, 132582006013344], [8411982045465752, 120265228895938], [-202130929834346534, -134759789971464], [497959039670431626, 7070249734188], [258852706693307532, -134935123551767], [-20610258718931890, 298067518462980], [-441622796386941988, -136515232021467], [-26986772505982590, -7456390721588], [-26537135073560342, 138030590259760], [411929877302719616, 37084960664416], [-614022970315827300, 57235894897909], [-386048902763918114, -203525562109548], [423230476833154662, -133363136644716], [388042080154852382, -208393040792568], [113234611690095364, -202575896752477], [-204333321576565142, 128728853864492], [48571314950381398, -311643696054328], [-696519890853548612, -113485563680755], [23602343563066978, 149180231535020], [158985639573524470, 312943163953476], [-47381914706624012, 70602866092311], [963602231460996106, -43558976951400], [706456288613795168, 102568996509896], [353799948592623442, -243209765655328], [-292863843870784424, 409452743267346], [-689339578642070268, -266907041178141], [-85337603248027258, 569159241841988], [556769493688804378, 272440795226488], [163830998858021068, -490543363200071], [213276629636169568, -321188224816248], [62753844948931784, 64455555022726], [731737768896939112, 50291970537854], [-75217566499241698, -437073739477628], [172475455350458764, 53401818554921], [707181932105748976, 374070962161876], [119326694381015666, -613508391368040], [620771214517967538, -248641626356340], [423625632151323448, -446752285882758], [-708104529178958114, -425988536389216], [-1000894156450023382, -33343756603892], [-48536235705730836, 208992947033297], [-511203740858205418, 232811250399560], [-1355412970257747660, 139768995404999], [1173119054697228342, -231382438444108], [1663157951968113818, 220585843852124], [-1354510604093122784, -331359849502248], [-775582540745081454, -250750983155012], [383401141617506628, 11414814712723], [-1082129010710294876, -266871489798885], [1777094908635869018, -18532937746960], [793130715668692234, -450937232010692], [-80570943881993072, 1037644280754956], [-377252873147341556, -77641814759159], [-812812820876837028, 364765319935205], [860634750879005090, -128285403375304], [1673616948336642900, -565367238775969], [1488689364252884822, -175542918406108], [-65741121594423640, 1471784912757614], [-425368043783647906, 300118110487860], [-1424211890246135760, 42086418281732], [-1080105354528563598, -677862132441956], [47746149097890718, -32761034589752], [-1296344785835464060, -115377839839101], [-594674622374267336, -1113275762112262], [1587063280185555578, 549200349126968], [664872131660768556, 990124658201505], [-2811819972002908992, 297123051694224], [1202225625079368386, 685063789425664], [-149835726856045170, -1035180311134144], [-1583663663614667404, 67762888290775], [-1504029184196343926, -842615035601808], [-2173289022768038, 721746254679852], [-3330677914828990240, -72273653015704], [455127600863109062, 589115638217232], [1666867371192479442, -864316915521064], [2018382295510327908, -992740807876853], [-472708543616904420, -1212561440459275], [190229761604668384, 1154932880322536], [-2071075659036372798, -439249326348644], [628009750933552534, -1124943956560060], [675423976883524410, -1595560883621684], [3532408503145755072, 820438565094992], [-1807163241954814884, -613877659241243], [2451500669572978956, -699713449640695], [3246705753266315172, 644840917045371], [-2165955850616575706, -970383782421004], [2714588608026486366, 1120820699599136], [-2306995756017947974, -563558395519656], [-522233807057515728, 136169383194948], [2039682187962963502, -551088988237788], [-3080493964462773354, -365878484114416], [-3258816335363594334, 801835051189772], [-855242316106171608, 612248282440718], [4123323128356595738, -943321299796708], [4510999569437402236, 424441790323677], [458756015211171006, -670497834365308], [1875748965738392300, 225971733655633], [-2094220310464626288, -610381930437364], [1248980683186471186, -1487523692090692], [1255830871804156452, -876661283842533], [-1241843124207306790, -1610896403868304], [2998110452259018890, 1255013604275900], [-1035315133466816496, -1781065584387348], [-3291001464233974450, -958581903950524], [-956754073021310858, -1938755206577048], [905020433765161504, -2010291130243368], [1387964061391833044, 872579534639327], [-1235249091463901650, 384491592883048], [1276925686985444136, 978485398911566], [4334833693886860766, -442845074266924], [-2194194838684120986, -2103767978926456], [-1367447728186053456, -2796475750802076], [1042124731617303524, -2759866655612437], [1996994465452092932, 2627365965142595], [3380664937407686394, 847900269906648], [5566792224054331242, 939919143204860], [4301027070832666856, -433660026117794], [2588223648376479754, -2405192395980304], [3700898711562074300, -2696445215835987], [5115875474109103788, 2592778906018337], [5258699287350745040, 1648246798527868], [-3119572879556951854, 3035434411051308], [-6648523087053202600, 279870195265362], [6867100928763893894, -1395272175478412], [5270613368483751372, 3127752065040857], [2147616304352004192, 1861884878791112], [6883910178043689826, 1744760531134848], [1535306541768586734, -803263930269912], [-4439172581839733206, -777285262757328], [-1020583025634212166, -1386340459291764], [-4326161940463200986, 3030009142939856], [9359838609078401592, -1530729000878406], [2364307886967235038, -2676248499811712], [6915967344327220676, -732754224294253], [2525187042464761080, -4000148338483126], [-215768143934533968, -3875432609898780], [-1999478103100681042, 1754004371463732], [3114264540419676284, -5348295252217827], [5642531048221484928, -3693331165465056], [-1837281210397798526, 2093818659925216], [6350512842019582754, -390289887221556], [-8981211414586435020, -1726310294695609], [1186514272015322292, 300936450049735], [-6798753637686462840, -409454077119850], [-6209488306220495798, -2459586503158736], [-1556720581813008864, 4761351043891032], [-4527619511772830970, 3113892368539728], [-3102795110626855316, -1246358741533423], [-12243987410422046062, -920897520815188], [-3263171846187628858, -1341797928057388], [4148577557793694398, 421676675690312], [5644882005086819216, -2626760299295860], [-8038227258970084106, -3638647162877616], [-1612722638199842302, -1287746553708932], [1871556763940850056, 2957105020392982], [-7868276735669570474, 3997914512844420], [3465289008834471342, 6595543482878504], [-5396163220078629740, 4185554258345903], [-2871800297348571876, 2242535895267125], [2864859224059903154, -66454655487264], [-4370007193833369742, 3009045829572076], [2368026696907844708, -2128848282311541], [-4216128115870634620, -1484331959396125], [-12289427901408452742, 2617154239021784], [604781858164132586, -4850442253556036], [-6970092575975375424, -2843011563903280], [-7610435746422737816, 3565706818122366], [1803172682877946254, 3836168955333544], [6177280173944045768, -1756693279281594], [6824995184575021402, -1414940720228], [9431202309910024574, -3855443335233260], [8912281886560552016, -3387350357628580], [-6742800297244956028, 21013947548515], [3881440022004980890, -5821757819003408], [18928431315697808842, 366984367799196], [-8645053754299030964, 5728053459490201], [-3985768351516770378, -3991609791232504], [4643552764743936540, 2968422069086837], [9777500095533701608, 2001540298941534]]
